/**********************************************************************
 *
 * libinterface.h -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: libinterface.h 172 1999-02-25 21:59:02Z rjmcnab $
 *
 *********************************************************************/


#ifndef LIBINTERFACE_H
#define LIBINTERFACE_H


#include "gsdlconf.h"
#include "browse.h"
#include "queryinfo.h"
#include "mgsearch.h"
#include "display.h"
#include "gdbmclass.h"
#include "gsdlunicode.h"

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace/std/map>
#elif defined(GSDL_USE_STL_H)
#  include <map.h>
#else
#  include <map>
#endif

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace/std/iostream>
#elif defined(GSDL_USE_IOS_H)
#  include <iostream.h>
#else
#  include <iostream>
#endif


#include "cgiargs.h"

#define MACROPRECEDENCE "style,collection,queryversion,version,language"


// errors related to this module
#define LI_NOERROR            0
#define LI_LOADDATABASEFAILED 1


class cfg_info_t {
public:
  cfg_info_t () {numbytes=0.0; numdocs=0.0;};

  text_t      maintainer;
//  text_tarray indexes;
  text_t      defaultindex;
  text_tarray macrofiles;
  text_t      builddate;
  text_tarray indexmap;
  text_tarray subcollectionmap;
  text_tarray languagemap;
  double      numbytes;
  double      numdocs;
  text_t      defaultaction;
  text_t      defaultpage;
  text_t      defaultencoding;
};


class libinterface {
public:
  libinterface ();
  virtual ~libinterface() {}

  // setgsdlhome will search for etc/collect.cfg and 
  // set collectdir and gdbmdir
  void setgsdlhome (const text_t &thegsdlhome);

  // collectdir contains etc/collect.cfg, archives,
  // index, and maybe a macros directory
  void setcollectdir (const text_t &thecollectdir);
  void setgdbmdir (const text_t &thegdbmdir);

  void sethttpprefix (const text_t &thehttpprefix);
  void setgwcgi (const text_t &thegwcgi);

  // init should be called after the various homes are set,
  // it returns 'false' on failure and 'true' on success
  bool init (ostream &logout);
  
  // examine the cgi arguments and create the appropriate page, 
  // outputing the page to textout and any debug information to logout
  //
  // returns LI_NOERROR on success, LI_LOADDATABASEFAILED on failure
  virtual int getpage (const text_t &argstr, ostream &textout, ostream &logout);
  
  // returns the compressed arguments for the last getpage
  const text_t &getlastcomparg() {return lastcomparg;}

  // returns the name of the collection
  virtual text_t get_collection_name() {return "";}


protected:
  cfg_info_t cfg_info;

  text_t gsdlhome;
  text_t gdbmdir;
  text_t collectdir;

  text_t books;
  text_t hrefs;
  text_t titles;
  vector<text_t> bookarr;
  vector<text_t> hrefarr;
  vector<text_t> titlearr;
  int numbooks;

  text_t text_default_index;
  text_t default_index;

  text_t httpprefix;
  text_t gwcgi;
  text_t quotedstring;

  text_t lastcomparg;

  gdbmclass gdbm;
  outconvertclass logconvert;
  displayclass disp;

  inconvertclass asciiinconvert;
  utf8inconvertclass utf8inconvert;
  mapinconvertclass gbinconvert;
  outconvertclass asciioutconvert;
  utf8outconvertclass utf8outconvert;
  mapoutconvertclass gboutconvert;


  browseclass *browse;
  mgsearchclass search;
  
  int cfg_read (const text_t &filename);

  virtual void get_arg_config (text_t &argconfigstr);
  text_t::iterator get_next_config_arg (text_t::iterator first, text_t::iterator last,
					text_t &argname, text_t &defaultvalue, 
					bool &longarg);

  virtual text_t get_compressed_args (cgiargsclass &args);
  virtual void expand_compressed_args (cgiargsclass &args);
  virtual void add_default_args (cgiargsclass &args);
  virtual void check_args (cgiargsclass &args);
  virtual void args_tounicode (cgiargsclass &args, inconvertclass &inconvert);

  virtual void define_general_macros (cgiargsclass &args, outconvertclass &outconvert, 
				      ostream &logout);

  virtual void set_selection_macro (const text_tarray &map, const text_t &indexarg,
				    const text_t &index);

  virtual void prepare_page (cgiargsclass &args, outconvertclass &outconvert,
			     ostream &logout);


  virtual void set_query_params (cgiargsclass &args, queryparamclass &queryparams);
  virtual void define_query_macros (cgiargsclass &args, 
				    queryparamclass &queryparams,
				    queryresultsclass &queryresults, 
				    ostream &logout);
  virtual void docLinks (cgiargsclass &args, queryresultsclass &queryresults, 
			 ostream &logout);

  virtual int query_action (cgiargsclass &args, outconvertclass &outconvert, 
			    ostream &textout, ostream &logout);
  virtual void browse_action (cgiargsclass &args, outconvertclass &outconvert, 
			      ostream &textout, ostream &logout);
  virtual void document_action (cgiargsclass &args, outconvertclass &outconvert, 
				ostream &textout, ostream &logout);
  virtual void page_action (cgiargsclass &args, outconvertclass &outconvert, 
			    ostream &textout, ostream &logout);
  virtual void auxiliary_action (cgiargsclass &args, outconvertclass &outconvert, 
				 ostream &textout, ostream &logout);
  
  virtual void displayresults (cgiargsclass &args, outconvertclass &outconvert, 
			       ostream &textout, ostream &logout,
			       queryresultsclass &queryresults);

  virtual void define_pref_macros (cgiargsclass &args, ostream &logout);
  virtual int do_query(cgiargsclass &args, queryparamclass &queryparams, 
			queryresultsclass &queryresults, ostream &logout);
  virtual void highlighttext(text_tarray &termvars, outconvertclass &outconvert, 
			     ostream &textout, ostream &logout, text_t &text);
  void format_querystring (text_t &querystring);


  virtual void define_collection_macros (cgiargsclass &/*args*/, ostream &/*logout*/) {}
  virtual bool collection_init(const text_t &/*collection*/) {return true;}
  virtual void displaydocsummary (cgiargsclass &/*args*/, outconvertclass &/*outconvert*/, 
				  ostream &/*textout*/, ostream &/*logout*/,
				  queryresultsclass &/*queryresults*/,
				  int /*resultnum*/) {}


};


#endif
